import os
import jinja2
from sendgrid.helpers.mail import (Mail, From, To, Subject, PlainTextContent,
    HtmlContent)
from email.message import EmailMessage
from tcw_tasks.templates import TEXT_TEMPLATE, HTML_TEMPLATE


class Formatter:
    def text_from_template(self, contest, winners):
        msg = jinja2.Template(TEXT_TEMPLATE).render(contest=contest,
            winners=winners)
        return msg.strip()

    def html_from_template(self, contest, winners):
        msg = jinja2.Template(HTML_TEMPLATE).render(contest=contest,
            winners=winners)
        return msg.strip()


class Message(Formatter):
    """
    Create email message for a finished contest
    """

    def __init__(self, *args, **kwargs):
        self.contest = None
        self.winners = []
        self.mail_from = os.getenv('TCW_MAIL_FROM', 'user@localhost')
        self.subject = 'Your Contest Kitty contest results'
        for k, v in kwargs.items():
            if hasattr(self, k):
                setattr(self, k, v)

        if self.contest is None:
            raise Exception('Contest object required')


    def get_message(self, local=True):
        if local:
            return self._get_email_msg()
        else:
            return self._get_sendgrid_msg()


    def _get_sendgrid_msg(self):
        msg = Mail(
            From(self.mail_from),
            To(self.contest.email),
            Subject(self.subject),
            PlainTextContent(self.text_from_template(self.contest, self.winners)),
            HtmlContent(self.html_from_template(self.contest, self.winners))
        )

        return msg


    def _get_email_msg(self):
        msg = EmailMessage()
        msg['Subject'] = self.subject
        msg['From'] = self.mail_from
        msg['To'] = self.contest.email

        msg.set_content(self.text_from_template(self.contest, self.winners))
        msg.add_alternative(self.html_from_template(self.contest, self.winners),
            subtype='html')

        return msg



import os
import sys
import time
import logging
import random
import datetime
import smtplib
import argparse
from sendgrid import SendGridAPIClient
from tcw.database import session, init_engine
from tcw.apps.contest.models import Fossil
from tcw_tasks.utils import expired_contests
from tcw_tasks.models import Message


# globals #
args = None
logger = logging.getLogger('tcw-tasks')


def main():
    parse_args()
    setup_logging()

    uri = os.getenv('SQLALCHEMY_DATABASE_URI', None)
    if not uri:
        logger.error('Must have SQLALCHEMY_DATABASE_URI environment var')
        sys.exit(1)

    init_engine(uri)
    logger.info("STARTING. Debug=%s" % args.debug)

    while True:
        finish_contests()
        logger.debug("Sleeping...")
        time.sleep(60)


def finish_contests():
    contests = []
    now = datetime.datetime.utcnow()

    try:
        contests = expired_contests()
        logger.info("%d contest(s) pending closure" % len(contests))
    except Exception as x:
        logger.debug(x)
        logger.debug("No contests pending closure")
        return

    for c in contests:
        try:
            winners = c.pick_winners()
            notify_owner(c, winners)
            logger.info("Owner of contest %s notified of the winners" % (c.name))

            f = Fossil(name=c.name)
            session.add(f)

            logger.info("Closing and removing contest (%s) %s" % (c.name, c.title))
            session.delete(c)
            session.commit()

        except Exception as x:
            logger.warning(x)
            session.rollback()


def notify_owner(contest, winners):
    local = not bool(os.getenv('SENDGRID_API_KEY', 0))
    msg = Message(contest=contest, winners=winners).get_message(local)
    if local:
        with smtplib.SMTP('localhost', 25) as s:
            s.send_message(msg)
    else:
        client = SendGridAPIClient(os.getenv('SENDGRID_API_KEY'))
        response = client.send(msg)


def setup_logging():
    level = logging.INFO

    if args.debug:
        level = logging.DEBUG

    logging.basicConfig(
        level=level,
        format='%(asctime)s|%(name)s|%(levelname)s|%(message)s'
    )


def parse_args():
    global args

    parser = argparse.ArgumentParser(
        description='TCWinners script. Handle contests and notify owners')
    parser.add_argument('-d','--debug',
        action='store_true',
        help='turn on debug messages')
    args = parser.parse_args()


if __name__ == '__main__':
    main()



import datetime
from sqlalchemy import select, func, or_
from tcw.database import session
from tcw.apps.contest.models import Contest, Entrant


def expired_contests():
    """
    Get list of contests whose time has expired, or has reached max number
    of entrants.
    """

    now = datetime.datetime.utcnow()

    #  subq = select(func.count(Contest.entrants)).where(
    #    Contest.id == Entrant.contest_id).scalar_subquery()

    # contests = session.query(Contest).filter(
    #    or_( Contest.expires < now, subq >= Contest.max_entrants )
    # ).all()

    contests = session.query(Contest).filter(Contest.expires < now).all()
    if contests is None or len(contests) == 0:
        raise Exception("No contests that meet criteria")

    return contests
